/*
 *  vector3D.cpp
 *  RayTracer
 *
 *
 *  Modified by Rick Skarbez for COMP 575 - Fall 2007
 *  Originally created by Eric Bennett on 8/8/06.
 */


#include "vector3D.h"

vector3D::vector3D()
{
	x = 0;
	y = 0;
	z = 0;
	isPoint = true;
}

vector3D::vector3D(double newx, double newy, double newz, bool newPoint)
{
	x = newx;
	y = newy;
	z = newz;
	isPoint = newPoint;
}

vector3D::vector3D(vector3D &v) 
{
	x = v.x; 
	y = v.y; 
	z = v.z;
}

double vector3D::dot(vector3D &B) 
{
	return (x*B.x + y*B.y + z*B.z);
}

double vector3D::dot(double Bx, double By, double Bz) 
{
	return (x*Bx + y*By + z*Bz);
}

double vector3D::dot(vector3D &A, vector3D &B) 
{
	return (A.x*B.x + A.y*B.y + A.z*B.z);
}

void vector3D::cross(vector3D &A, vector3D &B) 
{
	x = A.y*B.z - A.z*B.y;
	y = A.z*B.x - A.x*B.z;
	z = A.x*B.y - A.y*B.x;
}

double vector3D::length() 
{
	return sqrt(x*x + y*y + z*z);
}

void vector3D::normalize() 
{
	double t = x*x + y*y + z*z;
	if ((t != 0) && (t != 1.0))
		t = 1.0 / sqrt(t);
	x *= t;
	y *= t;
	z *= t;
}

void vector3D::normalize(vector3D &A) {
	double t = A.x*A.x + A.y*A.y + A.z*A.z;
	if ((t != 0) && (t != 1.0)) 
		t = 1.0 / sqrt(t);
	x = A.x*t;
	y = A.y*t;
	z = A.z*t;
}

void vector3D::scale(double s) {
	x *= s;
	y *= s;
	z *= s;
}

void vector3D::scale(vector3D &A, double s)
{
	x = A.x * s;
	y = A.y * s;
	z = A.z * s;
}

void vector3D::plus(vector3D &A) 
{
	set(x+A.x, y+A.y, z+A.z);
}

void vector3D::plus(vector3D &A, vector3D &B) 
{
	set(B.x+A.x, B.y+A.y, B.z+A.z);
}

void vector3D::minus(vector3D &A) 
{
	set(x-A.x, y-A.y, z-A.z);
}

void vector3D::minus(vector3D &A, vector3D &B) 
{
	set(A.x-B.x, A.y-B.y, A.z-B.z);
}

void vector3D::set(double newx, double newy, double newz)
{
	x = newx;
	y = newy;
	z = newz;
}

void vector3D::print()
{
	std::cout << "Vector Status X: " << x << " Y: " << y << " Z: " << z << "\n";
}

void vector3D::multiply(matrix44 &A)
{
	double q, oldX, oldY, oldZ;
	oldX = x; oldY = y; oldZ = z;
	
	double w = (isPoint) ? 1 : 0;
	x = A.m[0][0]*oldX + A.m[0][1]*oldY + A.m[0][2]*oldZ +A.m[0][3]*w;
	y = A.m[1][0]*oldX + A.m[1][1]*oldY + A.m[1][2]*oldZ +A.m[1][3]*w;
	z = A.m[2][0]*oldX + A.m[2][1]*oldY + A.m[2][2]*oldZ +A.m[2][3]*w;
	
	if(isPoint)
	{
		q = A.m[3][0]*oldX + A.m[3][1]*oldY + A.m[3][2]*oldZ +A.m[3][3]*w;
		x = x/q;
		y = y/q;
		z = z/q;
	}
}

void vector3D::multiply(matrix44 &A, vector3D &V)
{
	double q;
	double w = (V.isPoint) ? 1 : 0;
	x = A.m[0][0]*V.x + A.m[0][1]*V.y + A.m[0][2]*V.z +A.m[0][3]*w;
	y = A.m[1][0]*V.x + A.m[1][1]*V.y + A.m[1][2]*V.z +A.m[1][3]*w;
	z = A.m[2][0]*V.x + A.m[2][1]*V.y + A.m[2][2]*V.z +A.m[2][3]*w;
	
	if(V.isPoint)
	{
		q = A.m[3][0]*V.x + A.m[3][1]*V.y + A.m[3][2]*V.z +A.m[3][3]*w;
		x = x/q;
		y = y/q;
		z = z/q;
	}
}
